@@ -43,6 +43,7 @@
 #include <spa/param/audio/raw.h>
 #include <spa/param/profiler.h>
 #include <spa/pod/builder.h>
+#include <spa/pod/dynamic.h>
 #include <spa/support/plugin.h>
 #include <spa/utils/json.h>
 #include <spa/utils/names.h>
@@ -196,15 +197,18 @@ struct impl {
 
 	uint32_t max_buffer_size;
 	uint32_t buffer_delay;
+	uint32_t current_delay;
 
 	struct spa_handle *spa_handle;
 	struct spa_plugin_loader *loader;
+
+	bool monitor_mode;
 };
 
 static void process(struct impl *impl)
 {
 	struct pw_buffer *cout;
-	struct pw_buffer *pout;
+	struct pw_buffer *pout = NULL;
 	float rec_buf[impl->info.channels][impl->aec_blocksize / sizeof(float)];
 	float play_buf[impl->info.channels][impl->aec_blocksize / sizeof(float)];
 	float play_delayed_buf[impl->info.channels][impl->aec_blocksize / sizeof(float)];
@@ -218,7 +222,7 @@ static void process(struct impl *impl)
 	uint32_t rindex, pindex, oindex, pdindex, avail;
 	int32_t stride = 0;
 
-	if ((pout = pw_stream_dequeue_buffer(impl->playback)) == NULL) {
+	if (impl->playback != NULL && (pout = pw_stream_dequeue_buffer(impl->playback)) == NULL) {
 		pw_log_debug("out of playback buffers: %m");
 		goto done;
 	}
@@ -256,23 +260,50 @@ static void process(struct impl *impl)
 				impl->play_ringsize, pdindex % impl->play_ringsize,
 				(void *)play_delayed[i], size);
 
-		/* output to sink, just copy */
-		dd = &pout->buffer->datas[i];
-		memcpy(dd->data, play[i], size);
+		if (pout != NULL) {
+			/* output to sink, just copy */
+			dd = &pout->buffer->datas[i];
+			memcpy(dd->data, play[i], size);
 
-		dd->chunk->offset = 0;
-		dd->chunk->size = size;
-		dd->chunk->stride = stride;
+			dd->chunk->offset = 0;
+			dd->chunk->size = size;
+			dd->chunk->stride = stride;
+		}
 	}
 
 	spa_ringbuffer_read_update(&impl->rec_ring, rindex + size);
 	spa_ringbuffer_read_update(&impl->play_ring, pindex + size);
 	spa_ringbuffer_read_update(&impl->play_delayed_ring, pdindex + size);
 
-	pw_stream_queue_buffer(impl->playback, pout);
+	if (impl->playback != NULL)
+		pw_stream_queue_buffer(impl->playback, pout);
 
-	/* Now run the canceller */
-	spa_audio_aec_run(impl->aec, rec, play_delayed, out, size / sizeof(float));
+	if (SPA_UNLIKELY (impl->current_delay < impl->buffer_delay)) {
+		uint32_t delay_left = impl->buffer_delay - impl->current_delay;
+		uint32_t silence_size;
+
+		/* don't run the canceller until play_buffer has been filled,
+		 * copy silence to output in the meantime */
+		silence_size = SPA_MIN(size, delay_left * sizeof(float));
+		for (i = 0; i < impl->info.channels; i++)
+			memset(out[i], 0, silence_size);
+		impl->current_delay += silence_size / sizeof(float);
+		pw_log_debug("current_delay %d", impl->current_delay);
+
+		if (silence_size != size) {
+			const float *pd[impl->info.channels];
+			float *o[impl->info.channels];
+
+			for (i = 0; i < impl->info.channels; i++) {
+				pd[i] = play_delayed[i] + delay_left;
+				o[i] = out[i] + delay_left;
+			}
+			spa_audio_aec_run(impl->aec, rec, pd, o, size / sizeof(float) - delay_left);
+		}
+	} else {
+		/* run the canceller */
+		spa_audio_aec_run(impl->aec, rec, play_delayed, out, size / sizeof(float));
+	}
 
 	/* Next, copy over the output to the output ringbuffer */
 	avail = spa_ringbuffer_get_write_index(&impl->out_ring, &oindex);
@@ -479,14 +510,55 @@ static void input_param_latency_changed(struct impl *impl, const struct spa_pod
 	else
 		pw_stream_update_params(impl->capture, params, 1);
 }
-
-static void input_param_changed(void *data, uint32_t id, const struct spa_pod *param)
+static struct spa_pod* get_props_param(struct impl* impl, struct spa_pod_builder* b)
 {
-	struct impl *impl = data;
+	if (spa_audio_aec_get_params(impl->aec, NULL) > 0) {
+		struct spa_pod_frame f[2];
+		spa_pod_builder_push_object(
+		    b, &f[0], SPA_TYPE_OBJECT_Props, SPA_PARAM_Props);
+		spa_pod_builder_prop(b, SPA_PROP_params, 0);
+		spa_pod_builder_push_struct(b, &f[1]);
+
+		spa_audio_aec_get_params(impl->aec, b);
+
+		spa_pod_builder_pop(b, &f[1]);
+		return spa_pod_builder_pop(b, &f[0]);
+	}
+	return NULL;
+}
+
+static void input_param_changed(void *data, uint32_t id, const struct spa_pod* param)
+{
+	struct spa_pod_object* obj = (struct spa_pod_object*)param;
+	const struct spa_pod_prop* prop;
+	struct impl* impl = data;
 	switch (id) {
 	case SPA_PARAM_Latency:
 		input_param_latency_changed(impl, param);
 		break;
+	case SPA_PARAM_Props:
+		if (param != NULL) {
+			uint8_t buffer[1024];
+			struct spa_pod_dynamic_builder b;
+			const struct spa_pod* params[1];
+			SPA_POD_OBJECT_FOREACH(obj, prop)
+			{
+				if (prop->key == SPA_PROP_params) {
+					spa_audio_aec_set_params(impl->aec, &prop->value);
+				}
+			}
+
+			spa_pod_dynamic_builder_init(&b, buffer, sizeof(buffer), 4096);
+			params[0] = get_props_param(impl, &b.b);
+			if (params[0]) {
+				pw_stream_update_params(impl->capture, params, 1);
+				pw_stream_update_params(impl->playback, params, 1);
+			}
+			spa_pod_dynamic_builder_clean(&b);
+		} else {
+			pw_log_warn("param is null");
+		}
+		break;
 	}
 }
 
@@ -519,7 +591,11 @@ static void output_state_changed(void *data, enum pw_stream_state old,
 	switch (state) {
 	case PW_STREAM_STATE_PAUSED:
 		pw_stream_flush(impl->sink, false);
-		pw_stream_flush(impl->playback, false);
+		if (impl->playback != NULL)
+			pw_stream_flush(impl->playback, false);
+		if (old == PW_STREAM_STATE_STREAMING) {
+			impl->current_delay = 0;
+		}
 		break;
 	case PW_STREAM_STATE_UNCONNECTED:
 		pw_log_info("%p: output unconnected", impl);
@@ -548,17 +624,40 @@ static void output_param_latency_changed(struct impl *impl, const struct spa_pod
 
 	if (latency.direction == SPA_DIRECTION_INPUT)
 		pw_stream_update_params(impl->sink, params, 1);
-	else
+	else if (impl->playback != NULL)
 		pw_stream_update_params(impl->playback, params, 1);
 }
 
 static void output_param_changed(void *data, uint32_t id, const struct spa_pod *param)
 {
+	struct spa_pod_object *obj = (struct spa_pod_object *) param;
+	const struct spa_pod_prop *prop;
 	struct impl *impl = data;
 	switch (id) {
 	case SPA_PARAM_Latency:
 		output_param_latency_changed(impl, param);
 		break;
+	case SPA_PARAM_Props:
+		if (param != NULL) {
+			uint8_t buffer[1024];
+			struct spa_pod_dynamic_builder b;
+			const struct spa_pod* params[1];
+
+			SPA_POD_OBJECT_FOREACH(obj, prop)
+			{
+				if (prop->key == SPA_PROP_params) {
+					spa_audio_aec_set_params(impl->aec, &prop->value);
+				}
+			}
+			spa_pod_dynamic_builder_init(&b, buffer, sizeof(buffer), 4096);
+			params[0] = get_props_param(impl, &b.b);
+			if (params[0] != NULL) {
+				pw_stream_update_params(impl->capture, params, 1);
+				pw_stream_update_params(impl->playback, params, 1);
+			}
+			spa_pod_dynamic_builder_clean(&b);
+		}
+		break;
 	}
 }
 
@@ -635,8 +734,10 @@ static void sink_process(void *data)
 static void playback_destroy(void *d)
 {
 	struct impl *impl = d;
-	spa_hook_remove(&impl->playback_listener);
-	impl->playback = NULL;
+	if (impl->playback != NULL) {
+		spa_hook_remove(&impl->playback_listener);
+		impl->playback = NULL;
+	}
 }
 
 static const struct pw_stream_events playback_events = {
@@ -657,9 +758,9 @@ static int setup_streams(struct impl *impl)
 {
 	int res;
 	uint32_t n_params, i;
-	const struct spa_pod *params[1];
-	uint8_t buffer[1024];
-	struct spa_pod_builder b;
+	uint32_t offsets[512];
+	const struct spa_pod *params[512];
+	struct spa_pod_dynamic_builder b;
 	struct pw_properties *props;
 	const char *str;
 	uint32_t index;
@@ -681,6 +782,8 @@ static int setup_streams(struct impl *impl)
 		pw_properties_set(props, SPA_KEY_AUDIO_CHANNELS, str);
 	if ((str = pw_properties_get(impl->source_props, SPA_KEY_AUDIO_POSITION)) != NULL)
 		pw_properties_set(props, SPA_KEY_AUDIO_POSITION, str);
+	if ((str = pw_properties_get(impl->source_props, "resample.prefill")) != NULL)
+		pw_properties_set(props, "resample.prefill", str);
 
 	impl->capture = pw_stream_new(impl->core,
 			"Echo-Cancel Capture", props);
@@ -701,32 +804,38 @@ static int setup_streams(struct impl *impl)
 			&impl->source_listener,
 			&source_events, impl);
 
-	props = pw_properties_new(
+	if (impl->monitor_mode) {
+		impl->playback = NULL;
+	} else {
+		props = pw_properties_new(
 			PW_KEY_NODE_NAME, "echo-cancel-playback",
 			PW_KEY_NODE_VIRTUAL, "true",
 			PW_KEY_NODE_PASSIVE, "true",
 			NULL);
-	if ((str = pw_properties_get(impl->sink_props, PW_KEY_NODE_GROUP)) != NULL)
-		pw_properties_set(props, PW_KEY_NODE_GROUP, str);
-	if ((str = pw_properties_get(impl->sink_props, PW_KEY_NODE_LINK_GROUP)) != NULL)
-		pw_properties_set(props, PW_KEY_NODE_LINK_GROUP, str);
-	if ((str = pw_properties_get(impl->sink_props, PW_KEY_NODE_LATENCY)) != NULL)
-		pw_properties_set(props, PW_KEY_NODE_LATENCY, str);
-	else if (impl->aec->latency)
-		pw_properties_set(props, PW_KEY_NODE_LATENCY, impl->aec->latency);
-	if ((str = pw_properties_get(impl->sink_props, SPA_KEY_AUDIO_CHANNELS)) != NULL)
-		pw_properties_set(props, SPA_KEY_AUDIO_CHANNELS, str);
-	if ((str = pw_properties_get(impl->sink_props, SPA_KEY_AUDIO_POSITION)) != NULL)
-		pw_properties_set(props, SPA_KEY_AUDIO_POSITION, str);
+		if ((str = pw_properties_get(impl->sink_props, PW_KEY_NODE_GROUP)) != NULL)
+			pw_properties_set(props, PW_KEY_NODE_GROUP, str);
+		if ((str = pw_properties_get(impl->sink_props, PW_KEY_NODE_LINK_GROUP)) != NULL)
+			pw_properties_set(props, PW_KEY_NODE_LINK_GROUP, str);
+		if ((str = pw_properties_get(impl->sink_props, PW_KEY_NODE_LATENCY)) != NULL)
+			pw_properties_set(props, PW_KEY_NODE_LATENCY, str);
+		else if (impl->aec->latency)
+			pw_properties_set(props, PW_KEY_NODE_LATENCY, impl->aec->latency);
+		if ((str = pw_properties_get(impl->sink_props, SPA_KEY_AUDIO_CHANNELS)) != NULL)
+			pw_properties_set(props, SPA_KEY_AUDIO_CHANNELS, str);
+		if ((str = pw_properties_get(impl->sink_props, SPA_KEY_AUDIO_POSITION)) != NULL)
+			pw_properties_set(props, SPA_KEY_AUDIO_POSITION, str);
+		if ((str = pw_properties_get(impl->sink_props, "resample.prefill")) != NULL)
+			pw_properties_set(props, "resample.prefill", str);
 
-	impl->playback = pw_stream_new(impl->core,
-			"Echo-Cancel Playback", props);
-	if (impl->playback == NULL)
-		return -errno;
+		impl->playback = pw_stream_new(impl->core,
+				"Echo-Cancel Playback", props);
+		if (impl->playback == NULL)
+			return -errno;
 
-	pw_stream_add_listener(impl->playback,
-			&impl->playback_listener,
-			&playback_events, impl);
+		pw_stream_add_listener(impl->playback,
+				&impl->playback_listener,
+				&playback_events, impl);
+	}
 
 	impl->sink = pw_stream_new(impl->core,
 			"Echo-Cancel Sink", impl->sink_props);
@@ -739,9 +848,23 @@ static int setup_streams(struct impl *impl)
 			&sink_events, impl);
 
 	n_params = 0;
-	spa_pod_builder_init(&b, buffer, sizeof(buffer));
-	params[n_params++] = spa_format_audio_raw_build(&b, SPA_PARAM_EnumFormat,
-			&impl->info);
+	spa_pod_dynamic_builder_init(&b, NULL, 0, 4096);
+
+	offsets[n_params++] = b.b.state.offset;
+	spa_format_audio_raw_build(&b.b, SPA_PARAM_EnumFormat, &impl->info);
+
+	int nbr_of_external_props = spa_audio_aec_enum_props(impl->aec, 0, NULL);
+	if (nbr_of_external_props > 0) {
+		for (int i = 0; i < nbr_of_external_props; i++) {
+			offsets[n_params++] = b.b.state.offset;
+			spa_audio_aec_enum_props(impl->aec, i, &b.b);
+		}
+		get_props_param(impl, &b.b);
+	}
+
+	for (i = 0; i < n_params; i++) {
+		params[i] = spa_pod_builder_deref(&b.b, offsets[i]);
+	}
 
 	if ((res = pw_stream_connect(impl->capture,
 			PW_DIRECTION_INPUT,
@@ -749,33 +872,44 @@ static int setup_streams(struct impl *impl)
 			PW_STREAM_FLAG_AUTOCONNECT |
 			PW_STREAM_FLAG_MAP_BUFFERS |
 			PW_STREAM_FLAG_RT_PROCESS,
-			params, n_params)) < 0)
+			params, n_params)) < 0) {
+		spa_pod_dynamic_builder_clean(&b);
 		return res;
+	}
 
 	if ((res = pw_stream_connect(impl->source,
 			PW_DIRECTION_OUTPUT,
 			PW_ID_ANY,
 			PW_STREAM_FLAG_MAP_BUFFERS |
 			PW_STREAM_FLAG_RT_PROCESS,
-			params, n_params)) < 0)
+			params, n_params)) < 0) {
+		spa_pod_dynamic_builder_clean(&b);
 		return res;
+	}
 
 	if ((res = pw_stream_connect(impl->sink,
 			PW_DIRECTION_INPUT,
 			PW_ID_ANY,
-			PW_STREAM_FLAG_MAP_BUFFERS |
-			PW_STREAM_FLAG_RT_PROCESS,
-			params, n_params)) < 0)
+			impl->playback != NULL ?
+				PW_STREAM_FLAG_MAP_BUFFERS | PW_STREAM_FLAG_RT_PROCESS :
+				PW_STREAM_FLAG_AUTOCONNECT | PW_STREAM_FLAG_MAP_BUFFERS | PW_STREAM_FLAG_RT_PROCESS,
+			params, n_params)) < 0) {
+		spa_pod_dynamic_builder_clean(&b);
 		return res;
+	}
 
-	if ((res = pw_stream_connect(impl->playback,
+	if (impl->playback != NULL && (res = pw_stream_connect(impl->playback,
 			PW_DIRECTION_OUTPUT,
 			PW_ID_ANY,
 			PW_STREAM_FLAG_AUTOCONNECT |
 			PW_STREAM_FLAG_MAP_BUFFERS |
 			PW_STREAM_FLAG_RT_PROCESS,
-			params, n_params)) < 0)
+			params, n_params)) < 0) {
+		spa_pod_dynamic_builder_clean(&b);
 		return res;
+	}
+
+	spa_pod_dynamic_builder_clean(&b);
 
 	impl->rec_ringsize = sizeof(float) * impl->max_buffer_size * impl->info.rate / 1000;
 	impl->play_ringsize = sizeof(float) * ((impl->max_buffer_size * impl->info.rate / 1000) + impl->buffer_delay);
@@ -963,6 +1097,10 @@ int pipewire__module_init(struct pw_impl_module *module, const char *args)
 		goto error;
 	}
 
+	impl->monitor_mode = false;
+	if ((str = pw_properties_get(props, "monitor.mode")) != NULL)
+		impl->monitor_mode = pw_properties_parse_bool(str);
+
 	impl->module = module;
 	impl->context = context;
 
@@ -986,13 +1124,23 @@ int pipewire__module_init(struct pw_impl_module *module, const char *args)
 		pw_properties_set(impl->source_props, PW_KEY_NODE_DESCRIPTION, "Echo-Cancel Source");
 	if (pw_properties_get(impl->source_props, PW_KEY_MEDIA_CLASS) == NULL)
 		pw_properties_set(impl->source_props, PW_KEY_MEDIA_CLASS, "Audio/Source");
+	if (pw_properties_get(impl->source_props, "resample.prefill") == NULL)
+		pw_properties_set(impl->source_props, "resample.prefill", "true");
 
 	if (pw_properties_get(impl->sink_props, PW_KEY_NODE_NAME) == NULL)
 		pw_properties_set(impl->sink_props, PW_KEY_NODE_NAME, "echo-cancel-sink");
 	if (pw_properties_get(impl->sink_props, PW_KEY_NODE_DESCRIPTION) == NULL)
 		pw_properties_set(impl->sink_props, PW_KEY_NODE_DESCRIPTION, "Echo-Cancel Sink");
 	if (pw_properties_get(impl->sink_props, PW_KEY_MEDIA_CLASS) == NULL)
-		pw_properties_set(impl->sink_props, PW_KEY_MEDIA_CLASS, "Audio/Sink");
+		pw_properties_set(impl->sink_props, PW_KEY_MEDIA_CLASS,
+				impl->monitor_mode ? "Stream/Input/Audio" : "Audio/Sink");
+	if (pw_properties_get(impl->sink_props, "resample.prefill") == NULL)
+		pw_properties_set(impl->sink_props, "resample.prefill", "true");
+	if (impl->monitor_mode) {
+		pw_properties_set(impl->sink_props, PW_KEY_NODE_PASSIVE, "true");
+		pw_properties_set(impl->sink_props, PW_KEY_STREAM_MONITOR, "true");
+		pw_properties_set(impl->sink_props, PW_KEY_STREAM_CAPTURE_SINK, "true");
+	}
 
 	if ((str = pw_properties_get(props, "aec.method")) != NULL)
 		pw_log_warn("aec.method is not supported anymore use library.name");
